/*
 Tecnisa - Tecnologia em Informtica

 Nome: CONSULTA_ERRO_FISCAL

  Descrio:
  Funo para verificar erros na regra fiscal.

 Criao:
 23/05/2012 - Marcelo Alexsandro Souza Silva
*/

CREATE TYPE ERRO_REGRAFISCAL_TYPE AS
OBJECT (CDREGRAFISC      VARCHAR2(10),
        CDIMPOSTO        VARCHAR2(2),
        CDIMPOSTONFE     VARCHAR2(2),
        CDSITUTRIB       VARCHAR2(2),
        CDFISCAL         VARCHAR2(1),
        VRALIQUOTA       FLOAT(126),
        CDFISCOPER       VARCHAR2(10),
        IDCADPADRAO      VARCHAR2(1),
        IDERRO           INTEGER)
/

CREATE TYPE ERRO_REGRAFISCAL_TABLE AS
TABLE OF ERRO_REGRAFISCAL_TYPE
/

CREATE OR REPLACE FUNCTION CONSULTA_ERRO_REGRAFISCAL
(P_CDREGRAFISC IN REGRAFISCAL.CDREGRAFISC%TYPE) RETURN ERRO_REGRAFISCAL_TABLE PIPELINED
IS
V_COUNT          INTEGER;
V_DTVALREGRAFISC DATE;
V_IDUTGRUPOFISCAL VARCHAR2(01);
OUT_RECORD ERRO_REGRAFISCAL_TYPE := ERRO_REGRAFISCAL_TYPE (NULL, NULL, NULL, NULL,
                                                           NULL, NULL, NULL, NULL, NULL);
BEGIN
   SELECT MAX(DTVALREGRAFISC)
     INTO V_DTVALREGRAFISC
     FROM REGRAFISCAL
    WHERE CDREGRAFISC = P_CDREGRAFISC;

   V_COUNT := 0;
   SELECT COUNT(1)
     INTO V_COUNT
     FROM VINCULOFISREGRA
    WHERE VINCULOFISREGRA.CDREGRAFISC = P_CDREGRAFISC;
   IF V_COUNT = 0 THEN
      OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
      OUT_RECORD.IDERRO      := 0; /*Nao ha vinculo algum para a respectiva regra fiscal*/
      PIPE ROW (OUT_RECORD);
   END IF;

   FOR C_IMPOSTO_OBRIGATORIO IN
      (SELECT IMPREGRAFISCAL.CDIMPOSTO, IMPOSTO.CDIMPOSTONFE
         FROM IMPREGRAFISCAL, IMPOSTO
        WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
          AND IMPREGRAFISCAL.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
          AND IMPREGRAFISCAL.IDIMPOBRIGREGRA = 'S'
          AND NOT EXISTS (SELECT 1
                            FROM DADOSFISCREGRA
                           WHERE DADOSFISCREGRA.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCREGRA.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
          AND NOT EXISTS (SELECT 1
                            FROM DADOSFISCNCM
                           WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCNCM.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
          AND NOT EXISTS (SELECT 1
                            FROM DADOSFISCPROD
                           WHERE DADOSFISCPROD.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCPROD.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
       ORDER BY IMPREGRAFISCAL.CDIMPOSTO)
   LOOP
      OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
      OUT_RECORD.CDIMPOSTO    := C_IMPOSTO_OBRIGATORIO.CDIMPOSTO;
      OUT_RECORD.CDIMPOSTONFE := C_IMPOSTO_OBRIGATORIO.CDIMPOSTONFE;
      OUT_RECORD.IDERRO       := 1; /*Imposto Obrigatorio sem dados fiscais*/
      PIPE ROW (OUT_RECORD);
   END LOOP;

   FOR C_CFOP_OBRIGATORIO IN
      (SELECT IMPOSTO.CDIMPOSTO, IMPOSTO.NMIMPOSTO, IMPOSTO.CDIMPOSTONFE
         FROM IMPREGRAFISCAL, IMPOSTO
        WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
          AND IMPREGRAFISCAL.CDIMPOSTO = IMPOSTO.CDIMPOSTO
          AND IMPREGRAFISCAL.IDIMPOBRIGREGRA = 'S'
          AND IMPOSTO.IDUTCFOPREGFISC = 'S'
          AND NOT EXISTS (SELECT 1
                            FROM CFOPNCMREGRAFIS
                           WHERE CFOPNCMREGRAFIS.CDREGRAFISC = P_CDREGRAFISC)
          AND NOT EXISTS (SELECT 1
                            FROM CFOPADRAOREGFIS
                           WHERE CFOPADRAOREGFIS.CDREGRAFISC = P_CDREGRAFISC)
          AND NOT EXISTS (SELECT 1
                            FROM CFOPREGRAFISCAL
                           WHERE CFOPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC)
       ORDER BY IMPOSTO.CDIMPOSTO)
   LOOP
      OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
      OUT_RECORD.CDIMPOSTO    := C_CFOP_OBRIGATORIO.CDIMPOSTO;
      OUT_RECORD.CDIMPOSTONFE := C_CFOP_OBRIGATORIO.CDIMPOSTONFE;
      OUT_RECORD.IDERRO    := 2; /*Imposto exige CFOP e a regra nao possui parametrizacao de CFOP*/
      PIPE ROW (OUT_RECORD);
   END LOOP;
   FOR C_CST_OBRIGATORIO IN
      (SELECT IMPOSTO.CDIMPOSTO, IMPOSTO.NMIMPOSTO, IMPOSTO.CDIMPOSTONFE
         FROM IMPREGRAFISCAL, IMPOSTO
        WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
          AND IMPREGRAFISCAL.CDIMPOSTO = IMPOSTO.CDIMPOSTO
          AND IMPREGRAFISCAL.IDIMPOBRIGREGRA = 'S'
          AND IMPOSTO.IDUTCSTREGFISC = 'S'
          AND (EXISTS    (SELECT 1
                            FROM DADOSFISCREGRA
                           WHERE DADOSFISCREGRA.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCREGRA.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO
                             AND DADOSFISCREGRA.CDSITUTRIB IS NULL)
               OR EXISTS (SELECT 1
                            FROM DADOSFISCPROD
                           WHERE DADOSFISCPROD.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCPROD.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO
                             AND DADOSFISCPROD.CDSITUTRIB IS NULL)
               OR EXISTS (SELECT 1
                            FROM DADOSFISCNCM
                           WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
                             AND DADOSFISCNCM.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO
                             AND DADOSFISCNCM.CDSITUTRIB IS NULL)
               )
       ORDER BY IMPOSTO.CDIMPOSTO)
   LOOP
      OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
      OUT_RECORD.CDIMPOSTO    := C_CST_OBRIGATORIO.CDIMPOSTO;
      OUT_RECORD.CDIMPOSTONFE := C_CST_OBRIGATORIO.CDIMPOSTONFE;
      OUT_RECORD.IDERRO       := 3; /*Imposto exige CST e existem dados fiscais (padrao e/ou por produto) sem CST*/
      PIPE ROW (OUT_RECORD);
   END LOOP;

   FOR C_DADOSFISCREGRA IN
      (SELECT IMPOSTO.CDIMPOSTONFE, IMPOSTO.CDIMPOSTO, DADOSFISCREGRA.CDSITUTRIB, DADOSFISCREGRA.CDFISCAL, IMPOSTO.IDUTVLRQTDEIMPO,
              DADOSFISCREGRA.CDNATUBCPCSTNFE, DADOSFISCREGRA.CDTPCRPICOSTNFE, DADOSFISCREGRA.VRALIQUOTA, DADOSFISCREGRA.CDNATUREC
         FROM DADOSFISCREGRA, IMPOSTO
        WHERE DADOSFISCREGRA.CDREGRAFISC = P_CDREGRAFISC
          AND DADOSFISCREGRA.CDIMPOSTO = IMPOSTO.CDIMPOSTO)
   LOOP
      IF C_DADOSFISCREGRA.CDSITUTRIB IS NOT NULL THEN
         IF C_DADOSFISCREGRA.CDIMPOSTONFE IS NOT NULL THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCREGRA.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCREGRA.CDSITUTRIB;
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTONFE := C_DADOSFISCREGRA.CDIMPOSTONFE;
               OUT_RECORD.CDIMPOSTO    := C_DADOSFISCREGRA.CDIMPOSTO;
               OUT_RECORD.CDSITUTRIB   := C_DADOSFISCREGRA.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO  := 'S';
               OUT_RECORD.IDERRO       := 4; /*CST no cadastrado para o imposto.*/
               PIPE ROW (OUT_RECORD);
            END IF;

            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCREGRA.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCREGRA.CDSITUTRIB
               AND ((IDUTICDFISCAL1 = 'S' AND C_DADOSFISCREGRA.CDFISCAL = '1') OR
                    (IDUTICDFISCAL2 = 'S' AND C_DADOSFISCREGRA.CDFISCAL = '2') OR
                    (IDUTICDFISCAL3 = 'S' AND C_DADOSFISCREGRA.CDFISCAL = '3'));
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCREGRA.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCREGRA.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCREGRA.CDSITUTRIB;
               OUT_RECORD.CDFISCAL    := C_DADOSFISCREGRA.CDFISCAL;
               OUT_RECORD.IDCADPADRAO := 'S';
               OUT_RECORD.IDERRO      := 5; /*Cdigo Fiscal invlido para o respectivo CST e imposto*/
               PIPE ROW (OUT_RECORD);
            END IF;

            IF C_DADOSFISCREGRA.IDUTVLRQTDEIMPO = 'S' THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO      = C_DADOSFISCREGRA.CDIMPOSTONFE
                  AND CDSITUTRIB     = C_DADOSFISCREGRA.CDSITUTRIB;
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCREGRA.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCREGRA.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCREGRA.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'S';
                  OUT_RECORD.IDERRO      := 6; /*CST invlido para imposto cujo valor de alquota  definido por quantidade.*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

            IF C_DADOSFISCREGRA.CDNATUBCPCSTNFE IS NULL OR
               C_DADOSFISCREGRA.CDTPCRPICOSTNFE IS NULL THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO       = C_DADOSFISCREGRA.CDIMPOSTONFE
                  AND CDSITUTRIB      = C_DADOSFISCREGRA.CDSITUTRIB
                  AND IDUTITIPCREDITO = 'N';
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCREGRA.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCREGRA.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCREGRA.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'S';
                  OUT_RECORD.IDERRO      := 7; /*Tipo de Credito e natureza da base de clculo obrigatorios para o respectivo imposto e CST*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

         END IF;
      END IF;

      IF (C_DADOSFISCREGRA.CDIMPOSTONFE = '50') OR (C_DADOSFISCREGRA.CDIMPOSTONFE = '60') THEN
         IF (C_DADOSFISCREGRA.CDSITUTRIB <> '05') AND (C_DADOSFISCREGRA.CDSITUTRIB <> '49') AND (C_DADOSFISCREGRA.CDSITUTRIB <> '99') THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM ALIQUOTACSTNFE
             WHERE CDIMPOSTO = C_DADOSFISCREGRA.CDIMPOSTONFE
               AND CDSITUTRIBNFE = C_DADOSFISCREGRA.CDSITUTRIB
               AND VRALIQUOTA = DECODE(C_DADOSFISCREGRA.CDFISCAL, '1', C_DADOSFISCREGRA.VRALIQUOTA, 0);
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCREGRA.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCREGRA.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCREGRA.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO := 'S';
               OUT_RECORD.IDERRO      := 8; /*Valor de aliquota invalida para o respectivo imposto e CST*/
               PIPE ROW (OUT_RECORD);
            END IF;
         END IF;
      END IF;
   END LOOP;

   FOR C_DADOSFISCPROD IN
      (SELECT IMPOSTO.CDIMPOSTONFE, IMPOSTO.CDIMPOSTO, DADOSFISCPROD.CDSITUTRIB, DADOSFISCPROD.CDFISCAL, IMPOSTO.IDUTVLRQTDEIMPO,
              DADOSFISCPROD.CDNATUBCPCSTNFE, DADOSFISCPROD.CDTPCRPICOSTNFE, DADOSFISCPROD.VRALIQUOTA, DADOSFISCPROD.CDNATUREC
         FROM DADOSFISCPROD, IMPOSTO
        WHERE DADOSFISCPROD.CDREGRAFISC = P_CDREGRAFISC
          AND DADOSFISCPROD.CDIMPOSTO = IMPOSTO.CDIMPOSTO)
   LOOP
      IF C_DADOSFISCPROD.CDSITUTRIB IS NOT NULL THEN
         IF C_DADOSFISCPROD.CDIMPOSTONFE IS NOT NULL THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCPROD.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCPROD.CDSITUTRIB;
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCPROD.CDIMPOSTONFE;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCPROD.CDIMPOSTO;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCPROD.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO := 'N';
               OUT_RECORD.IDERRO      := 4; /*CST no cadastrado para o imposto.*/
               PIPE ROW (OUT_RECORD);
            END IF;

            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCPROD.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCPROD.CDSITUTRIB
               AND ((IDUTICDFISCAL1 = 'S' AND C_DADOSFISCPROD.CDFISCAL = '1') OR
                    (IDUTICDFISCAL2 = 'S' AND C_DADOSFISCPROD.CDFISCAL = '2') OR
                    (IDUTICDFISCAL3 = 'S' AND C_DADOSFISCPROD.CDFISCAL = '3'));
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCPROD.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCPROD.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCPROD.CDSITUTRIB;
               OUT_RECORD.CDFISCAL    := C_DADOSFISCPROD.CDFISCAL;
               OUT_RECORD.IDCADPADRAO := 'N';
               OUT_RECORD.IDERRO      := 5; /*Cdigo Fiscal invlido para o respectivo CST e imposto*/
               PIPE ROW (OUT_RECORD);
            END IF;

            IF C_DADOSFISCPROD.IDUTVLRQTDEIMPO = 'S' THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO      = C_DADOSFISCPROD.CDIMPOSTONFE
                  AND CDSITUTRIB     = C_DADOSFISCPROD.CDSITUTRIB;
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCPROD.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCPROD.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCPROD.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'N';
                  OUT_RECORD.IDERRO      := 6; /*CST invlido para imposto cujo valor de alquota  definido por quantidade.*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

            IF C_DADOSFISCPROD.CDNATUBCPCSTNFE IS NULL OR
               C_DADOSFISCPROD.CDTPCRPICOSTNFE IS NULL THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO       = C_DADOSFISCPROD.CDIMPOSTONFE
                  AND CDSITUTRIB      = C_DADOSFISCPROD.CDSITUTRIB
                  AND IDUTITIPCREDITO = 'N';
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCPROD.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCPROD.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCPROD.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'N';
                  OUT_RECORD.IDERRO      := 7; /*Tipo de Credito e natureza da base de clculo obrigatorios para o respectivo imposto e CST*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

         END IF;
      END IF;

      IF (C_DADOSFISCPROD.CDIMPOSTONFE = '50') OR (C_DADOSFISCPROD.CDIMPOSTONFE = '60') THEN
         IF (C_DADOSFISCPROD.CDSITUTRIB <> '05') AND (C_DADOSFISCPROD.CDSITUTRIB <> '49') AND (C_DADOSFISCPROD.CDSITUTRIB <> '99') THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM ALIQUOTACSTNFE
             WHERE CDIMPOSTO = C_DADOSFISCPROD.CDIMPOSTONFE
               AND CDSITUTRIBNFE = C_DADOSFISCPROD.CDSITUTRIB
               AND VRALIQUOTA = DECODE(C_DADOSFISCPROD.CDFISCAL, '1', C_DADOSFISCPROD.VRALIQUOTA, 0);
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCPROD.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCPROD.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCPROD.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO := 'N';
               OUT_RECORD.IDERRO      := 8; /*Valor de aliquota invalida para o respectivo imposto e CST*/
               PIPE ROW (OUT_RECORD);
            END IF;
         END IF;
      END IF;
   END LOOP;

   FOR C_DADOSFISCNCM IN
      (SELECT IMPOSTO.CDIMPOSTONFE, IMPOSTO.CDIMPOSTO, DADOSFISCNCM.CDSITUTRIB, DADOSFISCNCM.CDFISCAL, IMPOSTO.IDUTVLRQTDEIMPO,
              DADOSFISCNCM.CDNATUBCPCSTNFE, DADOSFISCNCM.CDTPCRPICOSTNFE, DADOSFISCNCM.VRALIQUOTA, DADOSFISCNCM.CDNATUREC
         FROM DADOSFISCNCM, IMPOSTO
        WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
          AND DADOSFISCNCM.CDIMPOSTO = IMPOSTO.CDIMPOSTO)
   LOOP
      IF C_DADOSFISCNCM.CDSITUTRIB IS NOT NULL THEN
         IF C_DADOSFISCNCM.CDIMPOSTONFE IS NOT NULL THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCNCM.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCNCM.CDSITUTRIB;
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCNCM.CDIMPOSTONFE;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCNCM.CDIMPOSTO;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCNCM.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO := 'X';
               OUT_RECORD.IDERRO      := 4; /*CST no cadastrado para o imposto.*/
               PIPE ROW (OUT_RECORD);
            END IF;

            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM PARAMCST
             WHERE CDIMPOSTO  = C_DADOSFISCNCM.CDIMPOSTONFE
               AND CDSITUTRIB = C_DADOSFISCNCM.CDSITUTRIB
               AND ((IDUTICDFISCAL1 = 'S' AND C_DADOSFISCNCM.CDFISCAL = '1') OR
                    (IDUTICDFISCAL2 = 'S' AND C_DADOSFISCNCM.CDFISCAL = '2') OR
                    (IDUTICDFISCAL3 = 'S' AND C_DADOSFISCNCM.CDFISCAL = '3'));
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCNCM.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCNCM.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCNCM.CDSITUTRIB;
               OUT_RECORD.CDFISCAL    := C_DADOSFISCNCM.CDFISCAL;
               OUT_RECORD.IDCADPADRAO := 'X';
               OUT_RECORD.IDERRO      := 5; /*Cdigo Fiscal invlido para o respectivo CST e imposto*/
               PIPE ROW (OUT_RECORD);
            END IF;

            IF C_DADOSFISCNCM.IDUTVLRQTDEIMPO = 'S' THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO      = C_DADOSFISCNCM.CDIMPOSTONFE
                  AND CDSITUTRIB     = C_DADOSFISCNCM.CDSITUTRIB;
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCNCM.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCNCM.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCNCM.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'X';
                  OUT_RECORD.IDERRO      := 6; /*CST invlido para imposto cujo valor de alquota  definido por quantidade.*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

            IF C_DADOSFISCNCM.CDNATUBCPCSTNFE IS NULL OR
               C_DADOSFISCNCM.CDTPCRPICOSTNFE IS NULL THEN
               V_COUNT := 0;
               SELECT COUNT(1)
                 INTO V_COUNT
                 FROM PARAMCST
                WHERE CDIMPOSTO       = C_DADOSFISCNCM.CDIMPOSTONFE
                  AND CDSITUTRIB      = C_DADOSFISCNCM.CDSITUTRIB
                  AND IDUTITIPCREDITO = 'N';
               IF V_COUNT = 0 THEN
                  OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
                  OUT_RECORD.CDIMPOSTO   := C_DADOSFISCNCM.CDIMPOSTO;
                  OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCNCM.CDIMPOSTONFE;
                  OUT_RECORD.CDSITUTRIB  := C_DADOSFISCNCM.CDSITUTRIB;
                  OUT_RECORD.IDCADPADRAO := 'X';
                  OUT_RECORD.IDERRO      := 7; /*Tipo de Credito e natureza da base de clculo obrigatorios para o respectivo imposto e CST*/
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;

         END IF;
      END IF;

      IF (C_DADOSFISCNCM.CDIMPOSTONFE = '50') OR (C_DADOSFISCNCM.CDIMPOSTONFE = '60') THEN
         IF (C_DADOSFISCNCM.CDSITUTRIB <> '05') AND (C_DADOSFISCNCM.CDSITUTRIB <> '49') AND (C_DADOSFISCNCM.CDSITUTRIB <> '99') THEN
            V_COUNT := 0;
            SELECT COUNT(1)
              INTO V_COUNT
              FROM ALIQUOTACSTNFE
             WHERE CDIMPOSTO = C_DADOSFISCNCM.CDIMPOSTONFE
               AND CDSITUTRIBNFE = C_DADOSFISCNCM.CDSITUTRIB
               AND VRALIQUOTA = DECODE(C_DADOSFISCNCM.CDFISCAL, '1', C_DADOSFISCNCM.VRALIQUOTA, 0);
            IF V_COUNT = 0 THEN
               OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
               OUT_RECORD.CDIMPOSTO   := C_DADOSFISCNCM.CDIMPOSTO;
               OUT_RECORD.CDIMPOSTONFE:= C_DADOSFISCNCM.CDIMPOSTONFE;
               OUT_RECORD.CDSITUTRIB  := C_DADOSFISCNCM.CDSITUTRIB;
               OUT_RECORD.IDCADPADRAO := 'X';
               OUT_RECORD.IDERRO      := 8; /*Valor de aliquota invalida para o respectivo imposto e CST*/
               PIPE ROW (OUT_RECORD);
            END IF;
         END IF;
      END IF;
   END LOOP;

   FOR C_VINCULOFISREGRA IN
      (SELECT CDEMPRESA, IDENTRSAIDOP, CDTIPOOPER, CDINSCESTA, CDGRUPOFISC, NRORDPRIORIDADE, DTVALVINCFISC
         FROM VINCULOFISREGRA
        WHERE VINCULOFISREGRA.CDREGRAFISC = P_CDREGRAFISC)
   LOOP
      V_COUNT := 0;

      SELECT MAX(REGRAFISCAL.IDUTGRUPOFISCAL)
        INTO V_IDUTGRUPOFISCAL
        FROM REGRAFISCAL
        WHERE REGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC;

      IF V_IDUTGRUPOFISCAL = 'S'  THEN
         SELECT COUNT(1)
           INTO V_COUNT
           FROM VINCULOFISREGRA, REGRAFISCAL
          WHERE VINCULOFISREGRA.CDREGRAFISC     <> P_CDREGRAFISC
            AND VINCULOFISREGRA.CDEMPRESA       =  C_VINCULOFISREGRA.CDEMPRESA
            AND (NVL(VINCULOFISREGRA.IDENTRSAIDOP, 'T') = NVL(C_VINCULOFISREGRA.IDENTRSAIDOP, 'T'))
            AND (NVL(VINCULOFISREGRA.CDTIPOOPER  , 'T') = NVL(C_VINCULOFISREGRA.CDTIPOOPER, 'T'))
            AND (NVL(VINCULOFISREGRA.CDINSCESTA  , 'T') = NVL(C_VINCULOFISREGRA.CDINSCESTA, 'T'))
            AND VINCULOFISREGRA.CDREGRAFISC     = REGRAFISCAL.CDREGRAFISC
            AND REGRAFISCAL.IDATIVREGRAFISC     = 'S'
            AND (EXISTS (SELECT 1
                          FROM GRUPOFISCREGRA A, GRUPOFISCREGRA B
                         WHERE A.CDREGRAFISC = VINCULOFISREGRA.CDREGRAFISC
                           AND B.CDREGRAFISC = P_CDREGRAFISC
                           AND A.CDGRUPOFISC = B.CDGRUPOFISC))
            AND EXISTS (SELECT 1
                          FROM IMPREGRAFISCAL A, IMPREGRAFISCAL B
                         WHERE A.CDREGRAFISC = VINCULOFISREGRA.CDREGRAFISC
                           AND B.CDREGRAFISC = P_CDREGRAFISC
                           AND A.CDIMPOSTO   = B.CDIMPOSTO);
      ELSE
         SELECT COUNT(1)
           INTO V_COUNT
           FROM VINCULOFISREGRA, REGRAFISCAL
          WHERE VINCULOFISREGRA.CDREGRAFISC     <> P_CDREGRAFISC
            AND VINCULOFISREGRA.CDEMPRESA       =  C_VINCULOFISREGRA.CDEMPRESA
            AND (NVL(VINCULOFISREGRA.IDENTRSAIDOP, 'T') = NVL(C_VINCULOFISREGRA.IDENTRSAIDOP, 'T'))
            AND (NVL(VINCULOFISREGRA.CDTIPOOPER  , 'T') = NVL(C_VINCULOFISREGRA.CDTIPOOPER, 'T'))
            AND (NVL(VINCULOFISREGRA.CDINSCESTA  , 'T') = NVL(C_VINCULOFISREGRA.CDINSCESTA, 'T'))
            AND VINCULOFISREGRA.CDREGRAFISC     = REGRAFISCAL.CDREGRAFISC
            AND REGRAFISCAL.IDATIVREGRAFISC     = 'S'
            AND REGRAFISCAL.IDUTGRUPOFISCAL <> 'S'
            AND EXISTS (SELECT 1
                          FROM IMPREGRAFISCAL A, IMPREGRAFISCAL B
                         WHERE A.CDREGRAFISC = VINCULOFISREGRA.CDREGRAFISC
                           AND B.CDREGRAFISC = P_CDREGRAFISC
                           AND A.CDIMPOSTO   = B.CDIMPOSTO);
      END IF;

      IF V_COUNT > 0 THEN
         OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
         OUT_RECORD.IDERRO      := 9; /*Vinculo ja cadastrado para outra regra fiscal ativa*/
         PIPE ROW (OUT_RECORD);
         EXIT;
      END IF;
   END LOOP;

   V_COUNT := 0;
   SELECT COUNT(1)
     INTO V_COUNT
     FROM REGRAFISCAL
    WHERE REGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
      AND REGRAFISCAL.IDUTGRUPOFISCAL = 'N'
      AND EXISTS (SELECT 1
                    FROM GRUPOFISCREGRA
                   WHERE GRUPOFISCREGRA.CDREGRAFISC = P_CDREGRAFISC);
   IF V_COUNT > 0 THEN
      OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
      OUT_RECORD.IDERRO      := 11; /*Regra nao utiliza grupo fiscal porem, existem grupos fiscais parametrizados para a mesma*/
      PIPE ROW (OUT_RECORD);
   END IF;

   V_COUNT := 0;
   SELECT COUNT(1)
     INTO V_COUNT
     FROM REGRAFISCAL
    WHERE REGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
      AND REGRAFISCAL.IDUTGRUPOFISCAL = 'S'
      AND NOT EXISTS (SELECT 1
                        FROM GRUPOFISCREGRA
                       WHERE GRUPOFISCREGRA.CDREGRAFISC = P_CDREGRAFISC);
   IF V_COUNT > 0 THEN
      OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
      OUT_RECORD.IDERRO      := 12; /*Regra utiliza grupo fiscal porem, nao existem grupos fiscais parametrizados para a mesma*/
      PIPE ROW (OUT_RECORD);
   END IF;

   V_COUNT := 0;
   SELECT COUNT(1)
     INTO V_COUNT
     FROM IMPREGRAFISCAL
    WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
      AND CDIMPOSTO NOT IN (SELECT IMPREGRAFISCAL.CDIMPOSTO
                              FROM IMPREGRAFISCAL
                             WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
                               AND NOT EXISTS (SELECT 1
                                                 FROM DADOSFISCREGRA
                                                WHERE DADOSFISCREGRA.CDREGRAFISC = P_CDREGRAFISC
                                                  AND DADOSFISCREGRA.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
                               AND NOT EXISTS (SELECT 1
                                                 FROM DADOSFISCNCM
                                                WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
                                                  AND DADOSFISCNCM.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
                               AND NOT EXISTS (SELECT 1
                                                 FROM DADOSFISCPROD
                                                WHERE DADOSFISCPROD.CDREGRAFISC = P_CDREGRAFISC
                                                  AND DADOSFISCPROD.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO));


   IF V_COUNT = 0 THEN
      FOR C_IMPOSTO_OBRIGATORIO IN
         (SELECT IMPREGRAFISCAL.CDIMPOSTO, IMPOSTO.CDIMPOSTONFE
            FROM IMPREGRAFISCAL, IMPOSTO
           WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
             AND IMPREGRAFISCAL.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
             AND NOT EXISTS (SELECT 1
                               FROM DADOSFISCREGRA
                              WHERE DADOSFISCREGRA.CDREGRAFISC = P_CDREGRAFISC
                                AND DADOSFISCREGRA.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
             AND NOT EXISTS (SELECT 1
                               FROM DADOSFISCNCM
                              WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
                                AND DADOSFISCNCM.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
             AND NOT EXISTS (SELECT 1
                               FROM DADOSFISCPROD
                              WHERE DADOSFISCPROD.CDREGRAFISC = P_CDREGRAFISC
                                AND DADOSFISCPROD.CDIMPOSTO = IMPREGRAFISCAL.CDIMPOSTO)
          ORDER BY IMPREGRAFISCAL.CDIMPOSTO)
      LOOP
         OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
         OUT_RECORD.CDIMPOSTO    := C_IMPOSTO_OBRIGATORIO.CDIMPOSTO;
         OUT_RECORD.CDIMPOSTONFE := C_IMPOSTO_OBRIGATORIO.CDIMPOSTONFE;
         OUT_RECORD.IDERRO       := 13; /*Imposto sem dados fiscais*/
         PIPE ROW (OUT_RECORD);
      END LOOP;

      V_COUNT := 0;
      SELECT COUNT(1)
        INTO V_COUNT
        FROM IMPREGRAFISCAL
       WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC;
      IF V_COUNT = 0 THEN
         OUT_RECORD.CDREGRAFISC := P_CDREGRAFISC;
         OUT_RECORD.IDERRO      := 14; /*Regra sem impostos*/
         PIPE ROW (OUT_RECORD);
      END IF;
   END IF;
   FOR C_IMPOSTO_RANGE IN
      (SELECT IMPREGRAFISCAL.CDIMPOSTO
         FROM IMPREGRAFISCAL, IMPOSTO
        WHERE IMPREGRAFISCAL.CDREGRAFISC = P_CDREGRAFISC
          AND IMPREGRAFISCAL.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
          AND IMPOSTO.IDUTRANGEMB        = 'N'
          AND EXISTS (SELECT 1
                        FROM DADOSFISCNCM
                       WHERE DADOSFISCNCM.CDREGRAFISC = P_CDREGRAFISC
                         AND DADOSFISCNCM.CDIMPOSTO   = IMPREGRAFISCAL.CDIMPOSTO
                         AND DADOSFISCNCM.VRRANGINI IS NOT NULL)
       ORDER BY IMPREGRAFISCAL.CDIMPOSTO)
   LOOP
      OUT_RECORD.CDREGRAFISC  := P_CDREGRAFISC;
      OUT_RECORD.CDIMPOSTO    := C_IMPOSTO_RANGE.CDIMPOSTO;
      OUT_RECORD.IDERRO       := 15; /*Imposto cadastrado com range mas com o parmetro de utilizar range desmarcado*/
      PIPE ROW (OUT_RECORD);
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CONSULTA_ERRO_REGRAFISCAL','5.60.000'); END;
/
